/*
 *
 *   Copyright (c) International Business Machines  Corp., 2000
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or 
 *   (at your option) any later version.
 * 
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software 
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
*/ 
#ifndef _JFS_FS_I
#define _JFS_FS_I

#include <linux/jfs/jfs_xtree.h>
#include <linux/jfs/jfs_dtree.h>

typedef struct	jfs_rwlock {
	struct rw_semaphore rw_sem;
	atomic_t in_use;	/* for hacked implementation of trylock */
} jfs_rwlock_t;

#define JFS_IP(ip)	((struct jfs_inode_info *)(ip)->u.generic_ip)

struct jfs_inode_info {
	int	fileset;	/* 4: fileset number (always 16)*/
	uint	mode2;		/* 4: jfs-specific mode		*/
        pxd_t   ixpxd;		/* 8: inode extent descriptor	*/
	dxd_t	acl;		/* 16: dxd describing acl	*/
	dxd_t	ea;		/* 16: dxd describing ea	*/
	time_t	otime;		/* 4: time created	*/
	uint	next_index;	/* 4: next available directory entry index */
	int	acltype;	/* 4: Type of ACL	*/
	short	btorder;	/* 2: access order	*/
	short	btindex;	/* 2: btpage entry index*/
	struct inode *ipimap;	/* 4: inode map			*/
	ushort	flag;		/* 2: JFS in-memory flag*/
	unchar	cflag;		/* 1: commit flags		*/
	unchar	agno;		/* 1: ag number			*/
	ushort	bxflag;		/* 2: xflag of pseudo buffer?	*/
	short	blid;		/* 2: lid of pseudo buffer?	*/
	ushort	atlhead;	/* 2: anonymous tlock list head	*/
	ushort	atltail;	/* 2: anonymous tlock list tail	*/
	struct inode *atlnext;	/* 4: next inode w/anonymous txn's */
	struct inode *atlprev;	/* 4: previous inode w/anonymous txn's */
	struct page *extent_page; /* 4: page containing extent  */
	jfs_rwlock_t rdwrlock;	/* 12/20: read/write lock	*/
	ushort	xtlid;		/* 2: lid of xtree lock on directory */
	short	pad;		/* 2: pad			*/
	union {
		struct {
			xtpage_t _xtroot;	/* 288: xtree root */
			struct inomap *_imap;	/* 4: inode map header	*/
		} file;
		struct {
			dir_table_slot_t _table[12]; /* 96: directory index */
			dtroot_t _dtroot;	/* 288: dtree root */
		} dir;
		struct {
			unchar _unused[16];	/* 16: */
			dxd_t _dxd;		/* 16: */
			unchar _inline[128];	/* 128: inline symlink */
		} link;
	} u;
};
#define i_xtroot u.file._xtroot
#define i_imap u.file._imap
#define i_dirtable u.dir._table
#define i_dtroot u.dir._dtroot
#define i_inline u.link._inline

#endif
